/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.jboss.qa.brms.cameljboss.controllers;

import java.util.Collection;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.drools.KnowledgeBase;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;
import org.jboss.qa.brms.cameljboss.model.Person;
import org.jboss.qa.brms.cameljboss.tools.*;
import org.springframework.web.servlet.ModelAndView;

/**
 * web-app reproducer
 * 
 * 
 * @author rsynek
 */
public class CamelIntegrationController extends org.springframework.web.servlet.mvc.AbstractController {

    private final String VIEW = "index";

    private KnowledgeBase kbase;
    
    private StatefulKnowledgeSession session;

    private void configure(ICamelCommandExecutor executor) {
        try {
            session = kbase.newStatefulKnowledgeSession();
            executor.configure(session, "node1", "session", "direct:test-session");
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    /**
     * Scenario:
     * 1. inserts fact into stateful knowledge session and call fireAllRules
     * 2. tries to obtain the inserted object from the session through camel
     */
    private String runScenario(ICamelCommandExecutor executor) {
        try {
            FactHandle fh = executor.batchInsertFire(new Person("Karel", "films", 28));
            return "insert and fireAllRules finished OK";
            
          /* There is another issue for jaxb: https://bugzilla.redhat.com/show_bug.cgi?id=771203 , so obtaining object from
           * the session would not work.
            if(session.getObject(fh) != null) {
                return "getObject finished OK";
            } else {
                return "getObject finished with error";
            }*/
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        } finally {
            session.dispose();
        }
    }
    
    public void setKbase(KnowledgeBase kbase) {
        this.kbase = kbase;
    }

    @Override
    protected ModelAndView handleRequestInternal(HttpServletRequest hsr, HttpServletResponse hsr1) throws Exception {
        String[] values = hsr.getParameterValues("type");
        if(values == null || values.length != 1) {
            throw new IllegalArgumentException("type url param required (possible values: pojo, json, xstream, jaxb)");
        }

        ICamelCommandExecutor executor = getExecutor(values[0]);
        configure(executor);
        String msg = runScenario(executor);
        ModelAndView mav = new ModelAndView(VIEW);
        mav.addObject("msg", msg);
        return mav;
    }
    
    private ICamelCommandExecutor getExecutor(String type) {
        if("pojo".equals(type)) {
            return new PojoCommandExecutor();
        } else if("json".equals(type)) {
            return new JsonCommandExecutor();
        } else if("jaxb".equals(type)) {
            return new JaxbCommandExecutor();
        } else if("xstream".equals(type)) {
            return new XstreamCommandExecutor();   
        } else {
            throw new IllegalArgumentException("unsupported executor type: " + type);
        }
    }
    
}
