/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.jboss.qa.brms.cameljboss.tools;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.model.dataformat.JaxbDataFormat;
import org.drools.camel.component.DroolsPolicy;
import org.drools.command.Command;
import org.drools.command.CommandFactory;
import org.drools.runtime.ExecutionResults;
import org.drools.runtime.rule.FactHandle;
import org.drools.runtime.rule.QueryResults;
import org.jboss.qa.brms.cameljboss.model.Person;

/**
 * Sends commands to Camel route in XML format - using JAXB for marshalling and unmarshalling.
 * @author rsynek
 */
public class JaxbCommandExecutor extends AbstractCamelCommandExecutor {
    private static final String NEW_LINE = System.getProperty("line.separator");
    private String unmarshall = "direct:unmarshall";

    @Override
    protected RouteBuilder createRouteBuilder(final String fromExpression, final String nodeName, final String sessionName) {
        return new RouteBuilder() {

            @Override
            public void configure() throws Exception {
                JaxbDataFormat jdf = new JaxbDataFormat();
                jdf.setContextPath("org.jboss.qa.brms.cameljboss.model");
                jdf.setPrettyPrint(true);

                from(fromExpression).policy(new DroolsPolicy()).unmarshal(jdf).to("drools://" + nodeName + "/" + sessionName).marshal(jdf);
                from("direct:marshall").policy(new DroolsPolicy()).marshal(jdf);
                from("direct:unmarshall").policy(new DroolsPolicy()).unmarshal(jdf);
                from(LOOKUP_ENDPOINT).policy(new DroolsPolicy()).unmarshal(jdf).to("drools://" + nodeName).marshal(jdf);
            }
        };
    }

    @Override
    protected void init() { }

    @Override
    public FactHandle batchInsertFire(Person p) {
        List<Command> commands = new ArrayList<Command>();
        commands.add(CommandFactory.newInsert(p, "tempPerson"));
        commands.add(CommandFactory.newFireAllRules());
        
        String commandXml = template.requestBody("direct:marshall", CommandFactory.newBatchExecution(commands), String.class);
        
        String xml = template.requestBody(endpoint, commandXml, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(unmarshall, xml);

       return (FactHandle)res.getFactHandle("tempPerson");
    }

    @Override
    public Person getObject(FactHandle fh) {
        final String GET_OBJECT =
                "<batch-execution>" + NEW_LINE +
                "    <get-object fact-handle=\"" + fh.toExternalForm() + "\" out-identifier=\"tempPerson\"/>" + NEW_LINE +
                "</batch-execution>";

        String xml = template.requestBody(endpoint, GET_OBJECT, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(unmarshall, xml);
        return (Person) res.getValue("tempPerson");
    }

    @Override
    public Collection<?> getObjects() {
        final String GET_OBJECTS =
            "<batch-execution>" + NEW_LINE
            + "  <get-objects out-identifier=\"objects\"/>" + NEW_LINE
            +"</batch-execution>";
        String xml = template.requestBody(endpoint, GET_OBJECTS, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(unmarshall, xml);
        return (Collection<?>) res.getValue("objects");
    }

    @Override
    public void modify(Person p, FactHandle fh) {
        final String MODIFY =
                "<batch-execution>" + NEW_LINE
                + "<modify fact-handle=\"" + fh.toExternalForm() + "\">" + NEW_LINE
                + "    <set accessor=\"id\" value=\"" + p.getId() + "\"/>" + NEW_LINE
                + "    <set accessor=\"name\" value=\"" + p.getName() + "\"/>" + NEW_LINE
                + "    <set accessor=\"likes\" value=\"" + p.getLikes() + "\"/>" + NEW_LINE
                + "    <set accessor=\"age\" value=\"" + p.getAge() + "\"/>" + NEW_LINE
                + "</modify>" + NEW_LINE
                + "</batch-execution>";

        template.requestBody(endpoint, MODIFY, String.class);
    }

    @Override
    public void retract(FactHandle fh) {
        final String RETRACT =
            "<batch-execution>" + NEW_LINE
            + "  <retract fact-handle=\"" + fh.toExternalForm() + "\"/>" + NEW_LINE
            +"</batch-execution>";
        template.requestBody(endpoint, RETRACT);
    }

    @Override
    public Person getGlobal(String globalId) {
        final String GET_GLOBAL =
            "<batch-execution>" + NEW_LINE
            + "  <get-global identifier=\"" + globalId + "\" out-identifier=\"tempPerson\" />" + NEW_LINE
            + "</batch-execution>";
        String xml = template.requestBody(endpoint, GET_GLOBAL, String.class);
        ExecutionResults result = (ExecutionResults)template.requestBody(unmarshall, xml);
        return (Person) result.getValue("tempPerson");
    }

    @Override
    public void setGlobal(Person p, String globalId) {
        final String SET_GLOBAL =
            "<batch-execution>" + NEW_LINE +
            "<set-global identifier=\"" + globalId + "\" out-identifier=\"tempPerson\">" + NEW_LINE +
            "    <object xsi:type=\"person\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + NEW_LINE +
            "        <name>" + p.getName() + "</name>" + NEW_LINE +
            "        <likes>" + p.getLikes() + "</likes>" + NEW_LINE +
            "        <id>" + p.getId() + "</id>" + NEW_LINE +
            "        <age>" + p.getAge() + "</age>" + NEW_LINE +
            "    </object>" + NEW_LINE +
            "</set-global>" + NEW_LINE +
            "</batch-execution>";
        template.requestBody(endpoint, SET_GLOBAL, String.class);
    }

    @Override
    public void startProcess(String processId) {
        final String START_PROCESS =
            "<batch-execution>" + NEW_LINE
            + "  <start-process processId=\"" + processId + "\"/>" + NEW_LINE
            +"</batch-execution>";

        template.requestBody(endpoint, START_PROCESS, String.class);
    }

    @Override
    public void signalEvent(String type, String event) {
        final String SIGNAL_EVENT =
            "<batch-execution>" + NEW_LINE +
            "<signal-event event-type=\"" + type + "\">" + NEW_LINE +
      //      "   <event xsi:type=\"string\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + NEW_LINE +
            "   <event>" + NEW_LINE +    
            "       event" + NEW_LINE +
            "   </event>" + NEW_LINE +
            "</signal-event>" + NEW_LINE +
            "</batch-execution>";

        template.requestBody(endpoint, SIGNAL_EVENT, String.class);
    }

    @Override
    public void insertElements(List<Person> objects) {
        String insertElements =
            "<batch-execution>" + NEW_LINE
            + "<insert-elements return-objects=\"true\">" + NEW_LINE
            + "<list>" + NEW_LINE;
        for(Person p : objects) {
            insertElements += "<element xsi:type=\"person\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + NEW_LINE
                + "<age>" + p.getAge() + "</age>" + NEW_LINE
                + "<id>" + p.getId() + "</id>" + NEW_LINE
                + "<likes>" + p.getLikes() + "</likes>" + NEW_LINE
                + "<name>" + p.getName() + "</name>" + NEW_LINE
                + "</element>" + NEW_LINE;
        }
        insertElements +=
            "</list>" + NEW_LINE
            + "</insert-elements>" + NEW_LINE
            + "</batch-execution>";

        template.requestBody(endpoint, insertElements, String.class);
    }

    @Override
    public QueryResults query(String queryName, Object[] args) {
        String queryStr =
            "<batch-execution>" + NEW_LINE
            + "<query out-identifier=\"persons\" name=\"" + queryName + "\">" + NEW_LINE;

        for(int i = 0; i < args.length; i++) {
            queryStr += serializeArg(args[i]);
        }
        queryStr +=
            "</query>" + NEW_LINE
            +"</batch-execution>";

        String xml = template.requestBody(endpoint, queryStr, String.class);
        ExecutionResults results = (ExecutionResults) template.requestBody(unmarshall, xml);
        return (QueryResults) results.getValue("persons");
    }

    private String serializeArg(Object arg) {
        if(arg instanceof Integer) {
            return "<arguments xsi:type=\"xs:int\" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + arg + "</arguments>";
        } else if (arg instanceof Boolean) {
            return "<arguments xsi:type=\"xs:boolean\" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + arg + "</arguments>";
        } else return "<arguments xsi:type=\"xs:string\" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + arg + "</arguments>";
    }

    @Override
    public void completeWorkItem(long id, Map<String, Object> results) {
        final String COMPLETE_WI =
            "<batch-execution>" + NEW_LINE
            + "  <complete-work-item id=\"" + id + "\"/>" + NEW_LINE
            + "</batch-execution>";
        template.requestBody(endpoint, COMPLETE_WI);
    }

    @Override
    public void abortWorkItem(long id) {
        final String ABORT_WI =
            "<batch-execution>" + NEW_LINE
            + "  <abort-work-item id=\"" + id + "\"/>" + NEW_LINE
            + "</batch-execution>";

        template.requestBody(endpoint, ABORT_WI);
    }

    @Override
    public void insertWithLookup(Person p) {
        final String LOOKUP_INSERT =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" + NEW_LINE +
            "   <batch-execution>" + NEW_LINE +
            "       <insert out-identifier=\"tempPerson\">" + NEW_LINE +
            "           <object xsi:type=\"person\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" + NEW_LINE +
            "               <name>" + p.getName() + "</name>" + NEW_LINE +
            "               <age>" + p.getAge() + "</age>" + NEW_LINE +
            "               <id>" + p.getId() + "</id>" + NEW_LINE +
            "               <likes>" + p.getLikes() + "</likes>" + NEW_LINE +
            "           </object>" + NEW_LINE +
            "       </insert>" + NEW_LINE +
            "</batch-execution>";

        template.requestBody(endpoint, LOOKUP_INSERT, String.class);
    }

}
