/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.jboss.qa.brms.cameljboss.tools;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.camel.builder.RouteBuilder;
import org.drools.camel.component.DroolsPolicy;
import org.drools.runtime.ExecutionResults;
import org.drools.runtime.rule.FactHandle;
import org.drools.runtime.rule.QueryResults;
import org.jboss.qa.brms.cameljboss.model.Person;

/**
 * Command execution through Camel using JSON command format
 * @author rsynek
 */
public class JsonCommandExecutor extends AbstractCamelCommandExecutor {

    private final String UNMARSHALL = "direct:unmarshall";

    @Override
    protected void init() {

    }

    @Override
    protected RouteBuilder createRouteBuilder(final String fromExpression, final String nodeName, final String sessionName) {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from(fromExpression).policy(new DroolsPolicy()).unmarshal("json").to("drools://" + nodeName + "/" + sessionName).marshal("json");
                from(UNMARSHALL).policy(new DroolsPolicy()).unmarshal("json");
                from("direct:marshall").policy(new DroolsPolicy()).marshal("json");
                from(LOOKUP_ENDPOINT).policy(new DroolsPolicy()).unmarshal("json").to("drools://" + nodeName).marshal("json");
            }
        };
    }

    @Override
    public FactHandle batchInsertFire(Person p) {
        final String BATCH_INSERT =
            "{"
            + "'batch-execution':{"
            + "    'commands':[{"
            + "        'insert':{"
            + "            'out-identifier':'tempPerson',"
            + "            'object':{"
            + "                'org.jboss.qa.brms.cameljboss.model.Person':{"
            + "                    'name':'" + p.getName() + "',"
            + "                    'age':"+ p.getAge() + ","
            + "                    'likes':'" + p.getLikes() + "',"
            + "                    'id':" + p.getId()
            + "                 }"
            + "             }"
            + "         },"
            + "         'fire-all-rules':''}]"
            + "     }"
            + "}";

        String json = template.requestBody(endpoint, BATCH_INSERT, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(UNMARSHALL, json);
        return (FactHandle) res.getFactHandle("tempPerson");
    }

    @Override
    public Person getObject(FactHandle fh) {
        final String GET_OBJECT =
            "{'batch-execution':{'commands':{'get-object':{'fact-handle':'" + fh.toExternalForm() + "','out-identifier':'tempPerson'}}}}";
        String json = template.requestBody(endpoint, GET_OBJECT, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(UNMARSHALL, json);
        return (Person) res.getValue("tempPerson");
    }

    @Override
    public Collection<?> getObjects() {
        final String GET_OBJECTS =
            "{'batch-execution':{'commands':{'get-objects':{'out-identifier':'objects'}}}}";

        String json = template.requestBody(endpoint, GET_OBJECTS, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(UNMARSHALL, json);
        return (Collection<?>) res.getValue("objects");
    }

    @Override
    public void modify(Person p, FactHandle fh) {
        final String MODIFY =
            "{'batch-execution':{'commands':{'modify':{"
            + "'fact-handle':'" + fh.toExternalForm() + "',"
            + "'setters':[{'accessor':'id','value':" + p.getId() + "},"
            + "{'accessor':'name','value':'" + p.getName() + "'},"
            + "{'accessor':'likes','value':'" + p.getLikes() + "'},"
            + "{'accessor':'age','value':" + p.getAge() + "}]"
            + "}}}}";
        template.requestBody(endpoint, MODIFY, String.class);
    }

    @Override
    public void retract(FactHandle fh) {
        final String RETRACT =
            "{'batch-execution':{'commands':{'retract':{'fact-handle':'" + fh.toExternalForm() + "'}}}}";
        template.requestBody(endpoint, RETRACT, String.class);
    }

    @Override
    public Person getGlobal(String globalId) {
        final String GET_GLOBAL =
            "{'get-global':{'identifier':'" + globalId + "','out-identifier':'tempPerson'}}";

        String json = template.requestBody(endpoint, GET_GLOBAL, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(UNMARSHALL, json);
        return (Person) res.getValue("tempPerson");
    }

    @Override
    public void setGlobal(Person p, String globalId) {
        final String SET_GLOBAL =
            "{'set-global':{'identifier':'" + globalId + "','out-identifier':'tempPerson',"
                + "'object':{"
                + "'org.jboss.qa.brms.cameljboss.model.Person':{"
                + "'name':'" + p.getName() + "',"
                + "'age':"+ p.getAge() + ","
                + "'likes':'" + p.getLikes() + "',"
                + "'id':" + p.getId()
                + "}"
                + "}"
                + "}}";
        template.requestBody(endpoint, SET_GLOBAL, String.class);
    }

    @Override
    public void startProcess(String processId) {
        final String START_PROCESS = "{'start-process':{'process-id':'" + processId + "'}}";
        template.requestBody(endpoint, START_PROCESS, String.class);
    }

    @Override
    public void signalEvent(String type, String event) {
        final String SIGNAL = "{'signal-event':{'event-type':'" + type + "','object':{'java.lang.String':'" + event + "'}}}";
        template.requestBody(endpoint, SIGNAL, String.class);
    }

    @Override
    public void insertElements(List<Person> objects) {
        String insElements = "{'insert-elements':{'objects':[";
        for(Person p : objects) {
             insElements +=  "{'containedObject':{"
                + "'@class':'org.jboss.qa.brms.cameljboss.model.Person',"
                     + "'name':'" + p.getName() + "',"
                     + "'age':" + p.getAge() + ","
                     + "'likes':'" + p.getLikes() + "',"
                     + "'id':" + p.getId() + "}},";
        }
        if(insElements.endsWith(",")) {
            insElements = insElements.substring(0, insElements.length()-1);
        }
        insElements += "]}}";
        template.requestBody(endpoint, insElements, String.class);
    }

    @Override
    public QueryResults query(String queryName, Object[] args) {

        String queryStr = "{'batch-execution':{'commands':{'query':{'out-identifier':'persons','name':'" + queryName + "', 'args':{";
        for(int i = 0; i < args.length; i++) {
            queryStr += serializeArg(args[i]) + ",";
        }
        if(queryStr.endsWith(",")) {
            queryStr = queryStr.substring(0, queryStr.length()-1);
        }
        queryStr += "}}}}}";

        String json = template.requestBody(endpoint, queryStr, String.class);
        ExecutionResults res = (ExecutionResults) template.requestBody(UNMARSHALL, json);
        return (QueryResults) res.getValue("persons");
    }

    private String serializeArg(Object arg) {
        if(arg instanceof Integer) {
            return "'int':" + arg;
        } else if (arg instanceof Boolean) {
            return "'boolean':" + arg;
        } else return "'string':'" + arg + "'";
    }

    @Override
    public void completeWorkItem(long id, Map<String, Object> results) {
         final String COMPLETE_WI = "{'complete-work-item':{'id':" + id + "}}";
         template.requestBody(endpoint, COMPLETE_WI, String.class);
    }

    @Override
    public void abortWorkItem(long id) {
        final String ABORT_WI = "{'abort-work-item':{'id':" + id + "}}";
        template.requestBody(endpoint, ABORT_WI, String.class);
    }

    @Override
    public void insertWithLookup(Person p) {
        final String LOOKUP_INSERT =
            "{"
            + "'batch-execution':{"
            + "    'commands':{"
            + "        'insert':{"
            + "            'out-identifier':'tempPerson',"
            + "            'object':{"
            + "                'org.jboss.qa.brms.cameljboss.model.Person':{"
            + "                    'name':'" + p.getName() + "',"
            + "                    'age':"+ p.getAge() + ","
            + "                    'likes':'" + p.getLikes() + "',"
            + "                    'id':" + p.getId()
            + "     }}}}}"
            + "}";

        template.requestBody(endpoint, LOOKUP_INSERT, String.class);
    }

}
