package org.jboss.qa.brms.cameljboss.tools;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.camel.builder.RouteBuilder;
import org.drools.command.BatchExecutionCommand;
import org.drools.command.Command;
import org.drools.command.CommandFactory;
import org.drools.command.Setter;
import org.drools.command.runtime.rule.GetObjectCommand;
import org.drools.command.runtime.rule.GetObjectsCommand;
import org.drools.runtime.ExecutionResults;
import org.drools.runtime.rule.FactHandle;
import org.drools.runtime.rule.QueryResults;
import org.jboss.qa.brms.cameljboss.model.Person;

/**
 * Sends commands to Camel route as POJO
 * @author rsynek
 */
public class PojoCommandExecutor extends AbstractCamelCommandExecutor {

    @Override
    protected void init() {

    }

    @Override
    public FactHandle batchInsertFire(Person p) {
        String outerId = "tempPerson";
        List<Command<?>> commands = new ArrayList<Command<?>>();
        commands.add(CommandFactory.newInsert(p, outerId));
        commands.add(CommandFactory.newFireAllRules());
        BatchExecutionCommand batchCommand = CommandFactory.newBatchExecution(commands, "ksession");
        ExecutionResults res = (ExecutionResults) template.requestBody(endpoint, batchCommand);
        return (FactHandle) res.getFactHandle(outerId);
    }

    @Override
    public void modify(Person p, FactHandle fh) {
        if(p == null)
            return;

        Setter ageSetter = CommandFactory.newSetter("age", String.valueOf(p.getAge()));
        Setter nameSetter = CommandFactory.newSetter("name", p.getName());
        Setter likesSetter = CommandFactory.newSetter("likes", p.getLikes());
        Setter idSetter = CommandFactory.newSetter("id", String.valueOf(p.getId()));

        List<Setter> setterList = new ArrayList<Setter>();
        setterList.add(ageSetter);
        setterList.add(nameSetter);
        setterList.add(idSetter);
        setterList.add(likesSetter);
        template.requestBody(endpoint, CommandFactory.newModify(fh, setterList));
    }

    @Override
    public Person getObject(FactHandle fh) {
        String outerId = "tempPerson";
        GetObjectCommand comm = (GetObjectCommand) CommandFactory.newGetObject(fh);
        comm.setOutIdentifier(outerId);
        ExecutionResults response = (ExecutionResults)template.requestBody(endpoint, comm);
        return (Person) response.getValue(outerId);
    }

    @Override
    public Collection<?> getObjects() {
        GetObjectsCommand objComm = (GetObjectsCommand) CommandFactory.newGetObjects();
        objComm.setOutIdentifier("objects");
        ExecutionResults result = (ExecutionResults)template.requestBody(endpoint, objComm);
        return (Collection<?>) result.getValue("objects");
    }

    @Override
    public void retract(FactHandle fh) {
        template.requestBody(endpoint, CommandFactory.newRetract(fh));
    }

    @Override
    public Person getGlobal(String globalId) {
        ExecutionResults result = (ExecutionResults)template.requestBody(endpoint, CommandFactory.newGetGlobal(globalId));
        return (Person)result.getValue(globalId);
    }

    @Override
    public void setGlobal(Person p, String globalId) {
        template.requestBody(endpoint, CommandFactory.newSetGlobal(globalId, p));
    }

    @Override
    public void startProcess(String processId) {
        template.requestBody(endpoint, CommandFactory.newStartProcess(processId));
    }

    @Override
    public void signalEvent(String type, String event) {
        template.requestBody(endpoint, CommandFactory.newSignalEvent(type, event));
    }

    @Override
    public QueryResults query(String queryName, Object[] args) {
        ExecutionResults res = (ExecutionResults) template.requestBody(endpoint, CommandFactory.newQuery("queryResId", queryName, args));
        return (QueryResults) res.getValue("queryResId");
    }

    @Override
    protected RouteBuilder createRouteBuilder(final String fromExpression, final String nodeName, final String sessionName) {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from(fromExpression).to("drools://" + nodeName + "/" + sessionName);
                from(LOOKUP_ENDPOINT).to("drools://" + nodeName);
            }
        };
    }

    @Override
    public void insertElements(List<Person> objects) {
        template.requestBody(endpoint, CommandFactory.newInsertElements(objects));
    }

    @Override
    public void completeWorkItem(long id, Map<String, Object> results) {
        template.requestBody(endpoint, CommandFactory.newCompleteWorkItem(id, results));
    }

    @Override
    public void abortWorkItem(long id) {
        template.requestBody(endpoint, CommandFactory.newAbortWorkItem(id));
    }

    @Override
    public void insertWithLookup(Person p) {
        List<Command<?>> commands = new ArrayList<Command<?>>();
        commands.add(CommandFactory.newInsert(p));
        template.requestBody(LOOKUP_ENDPOINT, CommandFactory.newBatchExecution(commands, "session"));
    }

}
