/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.jboss.qa.brms.cameljboss.tools;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.camel.builder.RouteBuilder;
import org.drools.camel.component.DroolsPolicy;
import org.drools.runtime.ExecutionResults;
import org.drools.runtime.rule.FactHandle;
import org.drools.runtime.rule.QueryResults;
import org.jboss.qa.brms.cameljboss.model.Person;

/**
 * Sends commands to Camel route as XML String - using XStream marshalling and unmarshalling.
 * Enables result unmarshalling through second route - "direct:unmarshall".
 * @author rsynek
 */
public class XstreamCommandExecutor extends AbstractCamelCommandExecutor implements ICamelCommandExecutor {
    private final String NEW_LINE = System.getProperty("line.separator");
    private String unmarshall;

    @Override
    protected RouteBuilder createRouteBuilder(final String fromExpression, final String nodeName, final String sessionName) {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from(fromExpression).policy(new DroolsPolicy()).unmarshal("xstream").to("drools://" + nodeName + "/" + sessionName).marshal("xstream");
                from("direct:unmarshall").policy(new DroolsPolicy()).unmarshal("xstream");
                from(LOOKUP_ENDPOINT).policy(new DroolsPolicy()).unmarshal("xstream").to("drools://" + nodeName).marshal("xstream");
            }
        };
    }

    @Override
    protected void init() {
        this.unmarshall = "direct:unmarshall";
    }

    @Override
    public FactHandle batchInsertFire(Person p) {
        final String BATCH_INSERT_FIRE =
                "<batch-execution>"
                + "   <insert out-identifier=\"tempPerson\">"
                + "       <org.jboss.qa.brms.cameljboss.model.Person>"
                + "           <name>" + p.getName() + "</name>"
                + "           <likes>" + p.getLikes() + "</likes>"
                + "           <id>" + p.getId() + "</id>"
                + "           <age>" + p.getAge() + "</age>"
                + "       </org.jboss.qa.brms.cameljboss.model.Person>"
                + "   </insert>"
                + "   <fire-all-rules />"
                + "</batch-execution>";

        String xml = template.requestBody(endpoint, BATCH_INSERT_FIRE, String.class);
        ExecutionResults response = (ExecutionResults) template.requestBody(unmarshall, xml);

        return (FactHandle) response.getFactHandle("tempPerson");
    }

    @Override
    public Person getObject(FactHandle fh) {
        final String GET_OBJECT =
                "<get-object fact-handle=\"" + fh.toExternalForm() + "\" out-identifier=\"tempPerson\"/>";
        String xml = template.requestBody(endpoint, GET_OBJECT, String.class);
        ExecutionResults response = (ExecutionResults) template.requestBody(unmarshall, xml);
        return (Person) response.getValue("tempPerson");
    }

    @Override
    public Collection<?> getObjects() {
        final String GET_OBJECTS = "<get-objects out-identifier=\"objects\">";
        String xml = template.requestBody(endpoint, GET_OBJECTS, String.class);
        ExecutionResults response = (ExecutionResults) template.requestBody(unmarshall, xml);
        return (Collection<?>) response.getValue("objects");
    }

    @Override
    public void modify(Person p, FactHandle fh) {
        final String MODIFY =
                "<modify fact-handle=\"" + fh.toExternalForm() + "\">"
                + "    <set accessor=\"id\" value=\"" + p.getId() + "\"/>"
                + "    <set accessor=\"name\" value=\"" + p.getName() + "\"/>"
                + "    <set accessor=\"likes\" value=\"" + p.getLikes() + "\"/>"
                + "    <set accessor=\"age\" value=\"" + p.getAge() + "\"/>"
                + "</modify>";

        template.requestBody(endpoint, MODIFY, String.class);
    }

    @Override
    public void retract(FactHandle fh) {
        final String RETRACT = "<retract fact-handle=\"" + fh.toExternalForm() + "\"/>";
        template.requestBody(endpoint, RETRACT);
    }

    @Override
    public Person getGlobal(String globalId) {
        final String GET_GLOBAL =
            "<get-global identifier=\"" + globalId + "\" out-identifier=\"tempPerson\" />";
        String xml = template.requestBody(endpoint, GET_GLOBAL, String.class);
        ExecutionResults result = (ExecutionResults)template.requestBody(unmarshall, xml);
        return (Person) result.getValue("tempPerson");
    }

    @Override
    public void setGlobal(Person p, String globalId) {
        final String SET_GLOBAL =
            "<set-global identifier=\"" + globalId + "\" out-identifier=\"tempPerson\">" +
            "    <org.jboss.qa.brms.cameljboss.model.Person>" +
            "        <name>" + p.getName() + "</name>" +
            "        <likes>" + p.getLikes() + "</likes>" +
            "        <id>" + p.getId() + "</id>" +
            "        <age>" + p.getAge() + "</age>" +
            "    </org.jboss.qa.brms.cameljboss.model.Person>" +
            "</set-global>";
        template.requestBody(endpoint, SET_GLOBAL, String.class);
    }

    @Override
    public void startProcess(String processId) {
        final String START_PROCESS =
            "<start-process processId=\"" + processId + "\"/>";

        template.requestBody(endpoint, START_PROCESS, String.class);
    }

    @Override
    public void signalEvent(String type, String event) {
        final String SIGNAL_EVENT =
            "<signal-event event-type=\"" + type + "\">" +
            "   <java.lang.String>" + event + "</java.lang.String>" +
            "</signal-event>";

        template.requestBody(endpoint, SIGNAL_EVENT, String.class);
    }

    @Override
    public void insertElements(List<Person> objects) {
        String insertStr = "<insert-elements>" + NEW_LINE;
        for(Person p : objects) {
            insertStr +=
            "    <org.jboss.qa.brms.cameljboss.model.Person>" + NEW_LINE +
            "        <name>" + p.getName() + "</name>" + NEW_LINE +
            "        <likes>" + p.getLikes() + "</likes>" + NEW_LINE +
            "        <id>" + p.getId() + "</id>" + NEW_LINE +
            "        <age>" + p.getAge() + "</age>" + NEW_LINE +
            "    </org.jboss.qa.brms.cameljboss.model.Person>" + NEW_LINE;
        }
        insertStr += "</insert-elements>";
        template.requestBody(endpoint, insertStr, String.class);
    }

    @Override
    public QueryResults query(String queryName, Object[] args) {
        String queryStr =
            "<query out-identifier=\"persons\" name=\"" + queryName + "\">" + NEW_LINE;

        for(int i = 0; i < args.length; i++) {
            queryStr += serializeArg(args[i]);
        }
        queryStr +=
            "</query>";

        String xml = template.requestBody(endpoint, queryStr, String.class);
        ExecutionResults results = (ExecutionResults) template.requestBody(unmarshall, xml);
        return (QueryResults) results.getValue("persons");
    }

    private String serializeArg(Object arg) {
        if(arg instanceof Integer) {
            return "<int>" + arg + "</int>";
        } else if (arg instanceof Boolean) {
            return "<boolean>" + arg + "</boolean>";
        } else return "<string>" + arg + "</string>";
    }

    @Override
    public void completeWorkItem(long id, Map<String, Object> results) {
        final String COMPLETE_WI =
            "<complete-work-item id=\"" + id + "\"/>";

        template.requestBody(endpoint, COMPLETE_WI);
    }

    @Override
    public void abortWorkItem(long id) {
        final String ABORT_WI =
            "<abort-work-item id=\"" + id + "\"/>";

        template.requestBody(endpoint, ABORT_WI);
    }

    @Override
    public void insertWithLookup(Person p) {
        final String BATCH_INSERT_FIRE =
                "<batch-execution lookup=\"session\">"
                + "   <insert out-identifier=\"tempPerson\">"
                + "       <org.jboss.qa.brms.cameljboss.model.Person>"
                + "           <name>" + p.getName() + "</name>"
                + "           <likes>" + p.getLikes() + "</likes>"
                + "           <id>" + p.getId() + "</id>"
                + "           <age>" + p.getAge() + "</age>"
                + "       </org.jboss.qa.brms.cameljboss.model.Person>"
                + "   </insert>"
                + "</batch-execution>";

        template.requestBody(endpoint, BATCH_INSERT_FIRE, String.class);
    }
}
